package com.xxfc.platform.activity.biz;

import com.github.wxiaoqi.security.admin.feign.dto.AppUserDTO;
import com.xxfc.platform.activity.constant.PrizeTypeEnum;
import com.xxfc.platform.activity.entity.ActivityWinningRecord;
import com.xxfc.platform.activity.util.LotteryUtils;
import com.xxfc.platform.activity.vo.ActivityPrizeVo;
import com.xxfc.platform.activity.vo.LotteryVo;
import lombok.RequiredArgsConstructor;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.xxfc.platform.activity.entity.ActivityPrize;
import com.xxfc.platform.activity.mapper.ActivityPrizeMapper;
import com.github.wxiaoqi.security.common.biz.BaseBiz;
import com.github.wxiaoqi.security.common.exception.BaseException;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 奖品设置
 *
 * @author libin
 * @email 18178966185@163.com
 * @date 2019-12-03 16:46:02
 */
@Transactional(rollbackFor = Exception.class)
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class ActivityPrizeBiz extends BaseBiz<ActivityPrizeMapper, ActivityPrize> {

    private final ActivityWinningRecordBiz activityWinningRecordBiz;
    private final UserCouponBiz userCouponBiz;

    /**
     * 查询奖品
     *
     * @return
     */
    public ActivityPrizeVo selectActivitys() {
        ActivityPrizeVo activityPrizeVo = new ActivityPrizeVo();
        List<ActivityPrize> activityPrizes = mapper.selectAll();
        if (CollectionUtils.isEmpty(activityPrizes)) {
            return activityPrizeVo;
        }
        Map<Integer, List<ActivityPrize>> activityPrizeMap = activityPrizes.stream().collect(Collectors.groupingBy(ActivityPrize::getType, Collectors.toList()));
        activityPrizeVo.setOnlinePrize(activityPrizeMap.get(PrizeTypeEnum.ONLINE.getCode()));
        activityPrizeVo.setLocalePrize(activityPrizeMap.get(PrizeTypeEnum.LOCALE.getCode()));
        return activityPrizeVo;
    }

    /**
     * 保存奖品设置
     *
     * @param activityPrizes
     */
    public void saveActivityPrizes(List<ActivityPrize> activityPrizes) {
        if (CollectionUtils.isEmpty(activityPrizes)) {
            throw new BaseException("奖品不能为空");
        }
        Map<Boolean, List<ActivityPrize>> activityPrizeMap = activityPrizes.stream().collect(Collectors.partitioningBy(x -> Objects.nonNull(x.getId()), Collectors.toList()));
        //保存
        List<ActivityPrize> activityPrizesOfSave = activityPrizeMap.get(Boolean.FALSE);
        if (CollectionUtils.isNotEmpty(activityPrizesOfSave)) {
            activityPrizesOfSave.stream().peek(x->x.setCrtTime(new Date())).count();
            mapper.insertList(activityPrizesOfSave);
        }
        //更新
        List<ActivityPrize> activityPrizesOfUpdate = activityPrizeMap.get(Boolean.TRUE);
        if (CollectionUtils.isNotEmpty(activityPrizesOfUpdate)) {
            for (ActivityPrize activityPrize : activityPrizesOfUpdate) {
                activityPrize.setUpdTime(new Date());
                mapper.updateByPrimaryKeySelective(activityPrize);
            }
        }
    }

    /**
     * 根据奖品类型查询
     *
     * @param prizeType
     * @return
     */
    public List<ActivityPrize> findActivityPrizeByType(Integer prizeType) {
        Example example = new Example(ActivityPrize.class);
        example.orderBy("serialNumber").asc();
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("type",prizeType);
        List<ActivityPrize> activityPrizes = mapper.selectByExample(example);
        if (CollectionUtils.isEmpty(activityPrizes)) {
            return Collections.EMPTY_LIST;
        }
        return activityPrizes;
    }



    /**
     * 抽奖
     *  //todo
     * @param activityId
     * @param prizeType
     * @param appUserDTO
     * @return
     */
    public LotteryVo activityLottery(Integer activityId, Integer prizeType, AppUserDTO appUserDTO) {
        LotteryVo lotteryVo = new LotteryVo();
        List<ActivityPrize> activityPrizes = findActivityPrizeByType(prizeType);
        int index = LotteryUtils.getrandomIndex(activityPrizes);
        ActivityPrize activityPrize = activityPrizes.get(index);
        if (activityPrize.getTotalStock() == 0){
            lotteryVo.setPrizeName("谢谢参与");
            return lotteryVo;
        }
        if (PrizeTypeEnum.LOCALE.getCode()==prizeType){
           //兑奖码生成
        }
        ActivityWinningRecord activityWinningRecord = new ActivityWinningRecord();
        activityWinningRecord.setActivityId(activityId);
        activityWinningRecord.setPrizeType(prizeType);
        activityWinningRecord.setUserId(appUserDTO.getUserid());
        activityWinningRecord.setIconPath(activityPrize.getIconPath());
        activityWinningRecordBiz.saveRecord(activityWinningRecord);
        return lotteryVo;
    }
}